"""Reverso Voice (voice.reverso.net) API for Python"""

from collections import namedtuple, defaultdict
import contextlib
import base64
import json
import io

import requests


__all__ = ["ReversoVoiceAPI", "Voice", "get_voices"]

BASE_URL = "https://voice.reverso.net/RestPronunciation.svc/v1/output=json/"

Voice = namedtuple("Voice", ("name", "language", "gender"))

def get_voices():
    _voices = defaultdict(list)
    for voice in json.loads(requests.get(BASE_URL + "GetAvailableVoices").content)["Voices"]:
        language_name = voice["Language"]
        _voices[language_name].append(
            Voice(voice["Name"], (int(voice["LangCode"]), language_name), voice["Gender"]))
    return dict(_voices)

class ReversoVoiceAPI:
    """Class for Reverso Voice API (https://voice.reverso.net/)

    Attributes:
        text
        voice
        speed
        mp3_data

    Methods:
        write_to_file(file)
        say(wait=False)

    """

    def __init__(self, text, voice, speed=100):
        self.__voice_names = [voice.name
                               for _, voices_list in get_voices().items()
                               for voice in voices_list]
        self.__text, self.__voice, self.__speed = None, None, None
        self.text, self.voice, self.speed = text, voice, speed
    
    @property
    def text(self):
        return self.__text

    @property
    def voice(self):
        return self.__voice

    @property
    def speed(self):
        return self.__speed

    @property
    def mp3_data(self):
        if self.__info_modified:
            self.__mp3_data = requests.get(BASE_URL + "GetVoiceStream/voiceName={}?voiceSpeed={}&inputText={}".format(self.voice, self.speed, base64.b64encode(self.text.encode()).decode())).content
            self.__info_modified = False
        return self.__mp3_data
    
    @text.setter
    def text(self, value):
        assert isinstance(value, str), "text must be a string"
        self.__text = value
        self.__info_modified = True

    @voice.setter
    def voice(self, value):
        if isinstance(value, Voice):
            value = value.name
        assert value in self.__voice_names, "invalid voice"
        self.__voice = value
        self.__info_modified = True

    @speed.setter
    def speed(self, value):
        assert isinstance(value, int), "speed must be an integer"
        assert 30 <= value <= 300, "speed must be 30 <= speed <= 300"
        self.__speed = value
        self.__info_modified = True
        

    def write_to_file(self, file):
        """Writes the spoken phrase to an MP3 file. You can specify either a filename-string or a file-like object.
        If you are trying to pass another object as a file argument, TypeError is raised.

        Args:
            file: The output file (both strings with filenames and file-like objects are supported)

        Returns:
            none

        Raises:
            TypeError: if not filename (string) or a file-like object is passed as a file argument.

        """

        if isinstance(file, str):
            with open(file, "wb") as fp:
                fp.write(self.mp3_data)
            return
        if hasattr(file, "write"):
            file.write(self.mp3_data)
            return
        raise TypeError("string or file-like object is required instead of {}".format(type(file)))

    def say(self, wait=False):
        """Reads the given text aloud. The difference from other methods is that this one PLAYS
        the sound (you can hear it if sound is enabled in your OS and pygame is installed).

        Note:
            Pygame is necessary for this method to work! You should install it before calling this
            method, otherwise ImportError will be raised.

        Args:
            wait: Tells whether it's necessary to wait until the text is fully spoken

        Returns:
            none

        Raises:
            ImportError: if Pygame is not installed

        """

        try:
            with contextlib.redirect_stdout(None):  # to remove the "Hello from the pygame community..." message
                import pygame
        except ImportError:
            raise ImportError("pygame is required for playing mp3 files, so you should install it first")

        pygame.mixer.init()
        with io.BytesIO() as fp:
            self.write_to_file(fp)
            fp.seek(0)
            pygame.mixer.music.load(fp)
            pygame.mixer.music.play()
            if wait:
                while pygame.mixer.music.get_busy():
                    pygame.time.delay(100)



import unittest
import contextlib

from reverso_api.voice import ReversoVoiceAPI, Voice, get_voices


def ask(what):
    while True:
        i = input("\n" + what + " [Y/N] ").upper()
        if i.startswith("Y"):
            return True
        elif i.startswith("N"):
            return False

class TestReversoVoiceAPI(unittest.TestCase):
    api = ReversoVoiceAPI("Hello, World!", "Heather22k")

    def test__get_voices(self):
        voices = get_voices()
        self.assertTrue(isinstance(voices, dict))
        for k, v in voices.items():
            self.assertTrue(isinstance(k, str))
            self.assertTrue(isinstance(v, list))
            for voice in v:
                self.assertTrue(isinstance(voice, Voice))              

    def test__mp3_data(self):
        voices = get_voices()
        for voice in (voices["US English"][0], "Heather22k"):
            self.api.voice = voice
            mp3_data = self.api.mp3_data
            self.assertTrue(isinstance(mp3_data, bytes))
            with open("data/voice/hello_world_us.mp3", "rb") as fp:
                self.assertEqual(fp.read(), mp3_data)

    def test__write_to_file(self):
        pass

    def test__say(self):
        voices = get_voices()
        try:
            with contextlib.redirect_stdout(None):
                import pygame
        except ImportError:
            print("\n.say(...) method cannot be checked without pygame. Install it first.")
            return
        for voice in [voices["US English"][0], "Heather22k"]:
            self.api.say(wait=True)
        self.assertTrue(ask("Have you heard the voice, played 2 times?"))



"""Reverso.net API for Python"""

from . import context, voice


__author__ = "Demian Wolf"
__copyright__ = "Copyright 2020, Demian Wolf"
__credits__ = ["Demian Wolf"]
__license__ = "MIT"
__version__ = "0.0.1.beta.3"
__maintainer__ = "Demian Wolf"
__email__ = "demianwolfssd@gmail.com"



"""Reverso Context (context.reverso.net) API for Python"""

from collections import namedtuple
import json

from bs4 import BeautifulSoup
import requests

__all__ = ["ReversoContextAPI", "WordUsageExample", "Translation", "InflectedForm"]

HEADERS = {"User-Agent": "Mozilla/5.0",
           "Content-Type": "application/json; charset=UTF-8"
           }

WordUsageExample = namedtuple("WordUsageExample",
                              ("text", "highlighted"))

Translation = namedtuple("Translation",
                         ("source_word", "translation", "frequency", "part_of_speech", "inflected_forms"))

InflectedForm = namedtuple("InflectedForm",
                           ("translation", "frequency"))


class ReversoContextAPI(object):
    """Class for Reverso Context API (https://voice.reverso.net/)

    Attributes:
        source_text
        target_text
        source_lang
        target_lang
        page_count
    
    Methods:
        get_translations()
        get_examples()

    """

    def __init__(self, source_text="пример", target_text="", source_lang="ru", target_lang="en"):
        self.__source_text, self.__target_text, self.__source_lang, self.__target_lang, self.__page_count = None, None, None, None, None
        self.source_text, self.target_text, self.source_lang, self.target_lang = source_text, target_text, source_lang, target_lang
        self.__update_data()
        
    def __update_data(self):
        self.__data = {
            "source_text": self.source_text,
            "target_text": self.target_text,
            "source_lang": self.source_lang,
            "target_lang": self.target_lang,
        }
        self.__info_modified = True

    @property
    def page_count(self):
        if self.__info_modified:
            self.__page_count = requests.post("https://context.reverso.net/bst-query-service", headers=HEADERS,
                                          data=json.dumps(self.__data)).json()["npages"]
            self.__info_modified = False
        return self.__page_count

    @property
    def source_text(self):
        return self.__source_text

    @property
    def target_text(self):
        return self.__target_text

    @property
    def source_lang(self):
        return self.__source_lang

    @property
    def target_lang(self):
        return self.__target_lang
    
    @source_text.setter
    def source_text(self, value):
        assert isinstance(value, str), "source text must be a string"
        self.__source_text = value
        self.__update_data()

    @target_text.setter
    def target_text(self, value):
        assert isinstance(value, str), "target text must be a string"
        self.__target_text = value
        self.__update_data()

    # TODO: add deeper check (is specified language really available) instead of just checking is a string given.

    @source_lang.setter
    def source_lang(self, value):
        assert isinstance(value, str), "language code must be a string"
        self.__source_lang = value
        self.__update_data()

    @target_lang.setter
    def target_lang(self, value):
        assert isinstance(value, str), "language code must be a string"
        self.__target_lang = value
        self.__update_data()

    def __repr__(self):
        return "ReversoContextAPI({source_text!r}, {target_text!r}, {source_lang!r}, {target_lang!r})" \
            .format(**self.__data)

    def __eq__(self, other):
        if isinstance(other, ReversoContextAPI):
            return self.source_text == other.source_text \
                   and self.target_text == other.target_text \
                   and self.source_lang == other.source_lang \
                   and self.target_lang == other.target_lang
        return False

    def get_translations(self):
        """Yields all available translations for the word (on the website you can find it just before the examples).

        Yields:
             Translation namedtuples.

        """

        translations_json = requests.post("https://context.reverso.net/bst-query-service", headers=HEADERS,
                                          data=json.dumps(self.__data)).json()["dictionary_entry_list"]
        for translation in translations_json:
            yield Translation(self.__data["source_text"], translation["term"], translation["alignFreq"],
                              translation["pos"],
                              [InflectedForm(form["term"], form["alignFreq"]) for form in
                               translation["inflectedForms"]])

    def get_examples(self):
        """A generator that gets words' usage examples pairs from server pair by pair.

        Note:
            Don't try to get all usage examples at one time if there are more than 5 pages (see the page_count attribute). It
            may take a long time to complete because it will be necessary to connect to the server as many times as there are pages exist.
            Just get the usage examples one by one as they are being fetched.

        Yields:
            Tuples with two WordUsageExample namedtuples (for source and target text and highlighted indexes)

        """

        def find_highlighted_idxs(soup, tag="em"):
            """Finds indexes of the parts of the soup surrounded by a particular HTML tag
            relatively to the soup without the tag.

            Example:
                soup = BeautifulSoup("<em>This</em> is <em>a sample</em> string")
                tag = "em"
                Returns: [(0, 4), (8, 16)]

            Args:
                soup: The BeautifulSoup's soup.
                tag: The HTML tag, which surrounds the parts of the soup.

            Returns:
                  A list of the tuples, which contain start and end indexes of the soup parts,
                  surrounded by tags.

            """

            cur, idxs = 0, []
            for t in soup.find_all(text=True):
                if t.parent.name == tag:
                    idxs.append((cur, cur + len(t)))
                cur += len(t)
            return idxs

        for npage in range(1, self.page_count + 1):
            self.__data["npage"] = npage
            examples_json = requests.post("https://context.reverso.net/bst-query-service", headers=HEADERS,
                                          data=json.dumps(self.__data)).json()["list"]
            for word in examples_json:
                source = BeautifulSoup(word["s_text"], features="lxml")
                target = BeautifulSoup(word["t_text"], features="lxml")
                yield (WordUsageExample(source.text, find_highlighted_idxs(source)),
                       WordUsageExample(target.text, find_highlighted_idxs(target)))



import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="Reverso_API",
    version="0.0.1.beta.3",
    license="MIT",
    author="Demian Wolf",
    author_email="demianwolfssd@gmail.com",
    description="Reverso.net API for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/demian-wolf/ReversoAPI",
    download_url="https://github.com/demian-wolf/ReversoAPI/archive/v0.0.1.beta.3.tar.gz",
    keywords=["REVERSO", "REVERSO-CONTEXT", "REVERSO CONTEXT", "CONTEXT", "REVERSO-VOICE", "REVERSO VOICE", "VOICE",
              "REVERSO-API", "API", "WRAPPER", "PYTHON"],
    packages=setuptools.find_packages(),
    install_requires=[
        "requests",
        "beautifulsoup4",
        "lxml",
    ],
    extras_require={
        "playing spoken text instead of just getting its MP3 data and/or saving it to file-like objects": ["pygame"],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
    ]
)
